# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import MessageV2, Worklist
from driverng.fixed import FixedLengthFormat, Multiple, String
from driverng.utils import num_years

SAMPLE_ID_LENGTH = 16


class AuQuery(FixedLengthFormat):
    FMT_DESCRIPTION = [
        # RB: Sample information request
        # R<space>: Normal Sample request
        # RH: Repeat run sample
        # Rh: Automatic repeat run sample
        # RE: Sample information request end
        String("code", 2),
        # 00 to 99
        String("unit_no", 2),
        # 4 ou 5 digits
        # 0001 to 9999
        String("rack", 4),
        # Rack sample: 01 to 10
        # STAT sample: 01 to 22
        String("cup", 2),
        # <space>: Serum
        # U: Urine
        # X: Other
        # Y: Other-1
        # W: Whole blood
        # N: Not specified
        String("sample_type", 1),
        # Routine sample: 0001 to 9999
        # Emergency sample: E001 to E999
        # STAT sample: P001 to P999
        # QC sample: Q001 to Q999
        # Reagent blank sample: R001 to R999
        # Calibration sample: A001 to A999
        String("sample_no", 4),
        String("sample_id", SAMPLE_ID_LENGTH, right_justified=True),
    ]


class AuOrder(FixedLengthFormat):
    HEADER = [
        String("rack", 4),
        String("cup", 2),
        String("sample_type", 1),
        String("sample_no", 4),
        String("sample_id", SAMPLE_ID_LENGTH, right_justified=True),
        String("dummy", 4),
        # 0 to 9 or E
        String("block_identification_number", 1),
    ]
    PATIENT = [
        # M: Male
        # F: Female
        # <space>: No reply
        # 0: none
        String("sex", 1),
        # 000 to 150
        # <space>: Not specified
        String("years_age", 3),
        # 00 to 11
        # <space>: Not specified
        String("months_age", 2),
        # 6 champs entre 0 et 20
        # On utilise que les deux premier pour nom et prenom
        String("patient_ipp", 20),
        String("patient_identity", 20, truncable=True),
    ]
    ORDER = [
        Multiple(
            "tests",
            [
                # 2 or 3 digits
                String("code", 3),
                # 0: Normal
                # 1: Diluent
                # 2: Condense
                String("diluent_type", 1),
            ],
        )
    ]

    FMT_DESCRIPTION = [String("code", 2)] + HEADER + PATIENT + ORDER


class AuRerunOrder(FixedLengthFormat):
    FMT_DESCRIPTION = [String("code", 2)] + AuOrder.HEADER + AuOrder.ORDER


class AuResult(FixedLengthFormat):
    TEST = [
        # 001 to 120
        # 3 ou 2 digits...
        String("test_number", 3),
        # Reagent (4x4)
        String("reagent", 16),
        # 9 ou 6 digits
        String("analysis_data", 9),
        # 2 ou 8 digits
        String("data_flags", 8),
    ]
    FMT_DESCRIPTION = (
        [
            # DB: Analysis data start
            # D<space>: Normal sample data
            # DH: Repeat run sample data
            # DR: Reagent blank sample data
            # DA: Calibration sample data
            # d : STAT quick output data
            # dH: Repeat run STAT quick output data
            # DQ: QC sample data
            # DE: Analysis data transmission end
            String("code", 2),
            String("unit_no", 2),
        ]
        + AuOrder.HEADER
        + AuOrder.PATIENT
        + [Multiple("tests", TEST)]
    )


class AuRerunResult(FixedLengthFormat):
    FMT_DESCRIPTION = (
        [String("code", 2), String("unit_no", 2)]
        + AuOrder.HEADER
        + [Multiple("tests", AuResult.TEST)]
    )


class AuQC(FixedLengthFormat):
    FMT_DESCRIPTION = (
        [
            # DQ: QC sample data
            String("code", 2),
            String("unit_no", 2),
        ]
        + AuOrder.HEADER
        + [Multiple("tests", AuResult.TEST)]
    )


class AuBaseDriver(MessageV2):
    @staticmethod
    def get_klass(code):
        if code in ("Sh", "SH"):
            return AuRerunOrder
        elif code[0] == "S":
            return AuOrder
        elif code[0] == "R":
            return AuQuery
        elif code in ("DQ", "DR", "DA"):
            return AuQC
        elif code in ("Dh", "DH"):
            return AuRerunResult
        elif code[0] == "D":
            return AuResult
        else:
            raise NotImplementedError

    @staticmethod
    def get_reagents(data):
        reagents = []
        for i in range(0, 16, 4):
            reagent = data[i:][:4]
            if reagent:
                reagents.append(reagent)
        return reagents

    @staticmethod
    def get_ise_reagents(data):
        reagents = []
        for i in range(0, 24, 8):
            reagent = data[i:][:8]
            if reagent:
                reagents.append(reagent.replace(" ", ""))
        return reagents

    @staticmethod
    def get_electrodes(data):
        electrodes = {}
        electrodes["Na Electrode Lot No."] = data[:5]
        electrodes["K Electrode Lot No."] = data[5:10]
        electrodes["Cl Electrode Lot No."] = data[10:15]
        electrodes["REF Electrode Lot No."] = data[15:20]
        return electrodes

    @staticmethod
    def get_query_worklist(message):
        worklist = Worklist(context="query")
        worklist.get_or_create(type="patient", sample_id=message.sample_id.strip())
        return worklist

    @staticmethod
    def get_result_worlist(message, worklist):
        rack = ""
        if message.rack and message.cup:
            rack = "{}-{}".format(message.rack, message.cup)
        elif message.rack and not message.cup:
            rack = message.rack
        return worklist.get_or_create(
            type="patient",
            sample_id=message.sample_id,
            specimen=message.sample_type or "S",
            rack=rack,
        )

    @staticmethod
    def get_qc_worklist(message, worklist):
        return worklist.get_or_create(
            type="control",
            control_lot=message.sample_id,
            sample_id=message.sample_id,
            specimen=message.sample_type or "S",
        )

    def create(self, code, **kwargs):
        kwargs.setdefault("code", code)
        return self.get_klass(code)(**kwargs)

    def parse(self, data):
        obj = self.get_klass(data[:2])()
        obj.parse(data)
        return obj

    def to_string(self, message):
        return message.to_string()

    def encode_worklist(self, worklist, context):
        samples = worklist.get_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: None},
        )
        if len(samples) == 0:
            if not context["query"]:
                return
            else:
                query = context["query"]
                return self.create(
                    "S{}".format(query.code[1:]),
                    sample_type=" " if query.sample_type == "N" else query.sample_type,
                    sample_id=query.sample_id,
                    block_identification_number="E",
                    rack=query.rack,
                    cup=query.cup,
                    sample_no=query.sample_no,
                )
        elif len(samples) > 1:
            raise RuntimeError("Multi-sample not implemented")
        else:
            sample = samples[0][0]

        if sample.specimen == "Serum" or sample.specimen == "S":
            sample_type = " "
        else:
            sample_type = sample.specimen

        query = context["query"]
        if query:
            code = "S{}".format(query.code[1:])
        else:
            code = "S"

        message = self.create(
            code,
            sample_type=sample_type,
            sample_id=sample.sample_id,
            block_identification_number="E",
        )
        if isinstance(message, AuOrder):
            message.sex = sample.sex if sample.sex in ("M", "F") else ""
            if sample.birthdate:
                message.years_age = "{:03d}".format(
                    num_years(sample.birthdate, sample.collection_date)
                )
            message.patient_ipp = sample.patient_id
            message.patient_identity = "{} {}".format(
                sample.last_name, sample.first_name
            )
        if query:
            message.rack = query.rack
            message.cup = query.cup
            message.sample_no = query.sample_no

        for order in sample.orders:
            message.tests.add(code=order.code, diluent_type="0")
        return message
