# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    HprimDate,
    HprimForeign,
    HprimMultipleList,
    HprimString,
    Segment,
)
from driverng.astm.parser import RecursiveList

logger = logging.getLogger("driverng.clinitek_advantus")


class ClinitekAdvantusSegmentH(HeaderSegment):
    manufacturer = HprimString((5, 1))
    analyser_code = HprimString((5, 2))
    analyser_serial = HprimString((5, 3))
    software_version = HprimString((5, 4))
    processing_id = HprimString(12)
    version_number = HprimString(13)
    message_date = HprimDate(14)

    patients = HprimForeign("P", "header")
    orders = HprimForeign("O", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class ClinitekAdvantusSegmentO(Segment):
    sample_id = HprimString(3)


class ClinitekAdvantusSegmentP(Segment):
    record_number = HprimString(2)
    test_sequence_number = HprimString(3)
    date = HprimDate(4)
    sample_id = HprimString(5)
    tech_id = HprimString(6)

    def to_string(self, separators=None):
        self.rang_segment = self.record_number
        return super(ClinitekAdvantusSegmentP, self).to_string(separators=separators)

    results = HprimForeign("R", "patient")


class ClinitekAdvantusSegmentPStrip(Segment):
    record_number = HprimString(2)
    results_format = HprimString(3)
    test_mode = HprimString(4)
    strip_id = HprimString(5)
    transfer_status = HprimString(6)
    error_code = HprimString(7)
    strip_lot_number = HprimString(8)
    strip_lot_expiration_date = HprimDate(9, fmt_date="%Y-%m")
    qc_lot_expiration_date = HprimDate(10, fmt_date="%Y-%m")

    def to_string(self, separators=None):
        self.rang_segment = self.record_number
        return super(ClinitekAdvantusSegmentPStrip, self).to_string(
            separators=separators
        )

    results = HprimForeign("R", "patient")


class ClinitekAdvantusSegmentR(Segment):
    result_status = HprimString(3)
    test_name = HprimString(4)
    value = HprimString((6, 1))
    units = HprimString((6, 2))
    clinical_level = HprimString(7)
    sample_notes = HprimMultipleList(11)


class ClinitekAdvantusSegmentL(Segment):
    termination_code = HprimString(3)


class Driver(AstmMessageV2):
    SPEC = {"code": "clinitek_advantus", "label": "Clinitek Advantus"}
    SEGMENT_CLASS_MAP = {
        "H": ClinitekAdvantusSegmentH,
        "O": ClinitekAdvantusSegmentO,
        "P": ClinitekAdvantusSegmentP,
        "R": ClinitekAdvantusSegmentR,
        "L": ClinitekAdvantusSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = ClinitekAdvantusSegmentH(type_segment="H", **kwargs)
        message.end = ClinitekAdvantusSegmentL(type_segment="L")
        return message

    def parse_segment(self, line, separators):
        segment = super(Driver, self).parse_segment(line, separators)
        if segment.type_segment == "P":
            if segment.record_number != "1":
                seg_p = ClinitekAdvantusSegmentPStrip()
                splitted = RecursiveList.split(line, separators)
                seg_p.populate(splitted)
                return seg_p

        return segment

    def decode_message(self, message):
        worklist = Worklist(context="result")

        for seg_p in message.patients:
            if seg_p.record_number == "1":
                sample_id = seg_p.sample_id
                date = seg_p.date
            if seg_p.record_number != "1":
                lots = []
                if seg_p.test_mode == "C":
                    sample = worklist.get_or_create(
                        control_lot=sample_id, sample_id=sample_id, type="control"
                    )
                    lots.append(
                        {
                            "lot": sample_id,
                            "expiration": seg_p.qc_lot_expiration_date,
                            "type": "control",
                        }
                    )
                if seg_p.test_mode == "P":
                    sample = worklist.get_or_create(sample_id=sample_id, type="patient")
                lots.append(
                    {
                        "lot": seg_p.strip_lot_number,
                        "expiration": seg_p.strip_lot_expiration_date,
                        "name": seg_p.strip_id,
                        "type": "reagent",
                    }
                )
                for seg_r in seg_p.results:
                    units = ""
                    comments = []
                    flags = []
                    if seg_r.units:
                        units = seg_r.units
                    if seg_r.sample_notes:
                        for note in seg_r.sample_notes:
                            comments.append(note)
                    if seg_r.clinical_level:
                        flags.append(seg_r.clinical_level)
                    sample.results.add(
                        code=seg_r.test_name,
                        value=seg_r.value,
                        date=date,
                        unit=units,
                        comments=comments,
                        flags=flags,
                        lots=lots,
                    )

        return worklist
